﻿using System;
using System.Collections.Generic;
using AppOptics.Instrumentation;
using Microsoft.AspNetCore;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Http;

namespace coretestapp
{

public class Startup
{
    string layerName = "example-layer";

    public void StartTrace()
    {
        System.Console.WriteLine("Create Start Trace Event");
        StartTraceEvent startEvent = Trace.StartTrace(layerName);
        startEvent.AddInfo("start-trace-event", "entry");
        startEvent.Report();
    }

    public void EndTrace()
    {
        System.Console.WriteLine("Create End Trace Event");
        EndTraceEvent endEvent = Trace.EndTrace(layerName);
        endEvent.AddInfo("end-trace-event", "exit");
        endEvent.Report();
    }

    public void TracedMethod()
    {
        System.Console.WriteLine("\n****** Enter Traced Method ***** \n");

        System.Console.WriteLine("Create Entry Event");
        EntryTraceEvent eventEntry = Trace.CreateEntryEvent("example-span-name");
        eventEntry.AddInfo("create-entry-event", "entry");
        eventEntry.Report();

        // Generate some info events
        for(int i = 0; i < 5; i++){
             InfoTraceEvent e = Trace.CreateInfoEvent("info_event_" + i);
             e.AddInfo("create-info-trace-event", "info_" + i);
             e.Report();
        }

        System.Console.WriteLine("Create Exit Event");
        ExitTraceEvent eventExit = Trace.CreateExitEvent("example-span-name");
        eventExit.AddInfo("create-exit-event", "exit");
        eventExit.Report();

        System.Console.WriteLine("\n****** Exit Traced Method ***** \n");
    }

    public void Configure(IApplicationBuilder app)
    { 
        // Add the AppOptics.Instrumentation Middleware
        app.UseAppopticsApm();

        app.Run(async context =>
        {
            // Using the AppOpticsApm Middleware will automatically start and end traces
            // and report metrics.  Alternatively, the reporting of StartTraceEvent and 
            // EndTraceEvents can be used as shown in the StartTrace and EndTrace methods.
            // As the AppOpticsApm Middleware is being used in this example those methods have
            // been commented out.

            // StartTrace();
            TracedMethod();
            await context.Response.WriteAsync("Hello, World!");
            // EndTrace();
        });
    }
}

public class Program
{
    public static void Main(string[] args)
    {
        System.Console.WriteLine("Running application ...");

        CreateWebHostBuilder(args).Build().Run();

        System.Console.WriteLine("Application has stopped.");
    }

    public static IWebHostBuilder CreateWebHostBuilder(string[] args) =>
        WebHost.CreateDefaultBuilder(args)
            .UseStartup<Startup>();
    }
}
